package HTFeed::PackageType::Google::Enqueue;

use HTFeed::DBTools qw(update_queue get_volumes_with_status);
use warnings;
use strict;
use LWP::UserAgent;
use Log::Log4perl qw(get_logger);

my $grin_convert_limit = 5000;
my $grin_query_limit = 100;

# This is the enqueuer for Google. It queries & updates GRIN to convert boooks
# & get the status of conversion requests.

# Find books that have been converted and are ready to download; update their status 
sub update_converted_books {
    my $ns_pkg = shift;
    my $grin_id = $ns_pkg->get('grinid');
    my $namespace = $ns_pkg->get_namespace();
    my $packagetype = $ns_pkg->get_packagetype();

    my $ready = 0;
    my $waiting = 0;
    my $error = 0;

    # get list of in-process books from DB
    my $in_process = get_volumes_with_status($packagetype,$namespace,'in_process'); 
    # query for status
    foreach my $grin_result ( @{ grin_query($grin_id,$in_process) } ) {
        my $objid = lc($grin_result->{objid});
        if($grin_result->{state} eq 'CONVERTED') {
            # ready for download
            update_queue($namespace,$objid,'ready'); 
            $ready++;
        } elsif($grin_result->{state} eq 'IN_PROCESS') {
            # still waiting
            $waiting++;
        } else {
            # if status is not IN_PROCESS or CONVERTED, set an error 
            update_queue($namespace,$objid,'punted');
            get_logger()->error('GRINError',objid=>$objid,namespace=>$namespace,field=>'grin_state',expected=>'CONVERTED',
                actual=>$grin_result->{state},detail => 'Unexpected GRIN state');
            $error++;
        }
    }
    get_logger->info("$namespace GRIN status: $ready ready; $waiting in process; $error errors");
}

sub convert_available_books {
    my $ns_pkg = shift;
    my $grin_id = $ns_pkg->get('grinid');
    my $namespace = $ns_pkg->get_namespace();
    my $packagetype = $ns_pkg->get_packagetype();

    my $actions = {
        'Success' => sub { update_queue(@_,'in_process') },
        'Already being converted' => sub { update_queue(@_,'in_process') },
        'Already available for download' => sub { update_queue(@_,'ready') },
    };

    my $available = get_volumes_with_status($packagetype,$namespace,'available');
    my $converted = 0;
    my $errors = 0;

    foreach my $convert_result ( @{ grin_convert($grin_id,$available) }) {
        my $result = $convert_result->{result};
        my $objid = $convert_result->{objid};
        if($actions->{$result}) {
            $converted++;
            &{$actions->{$result}}($namespace,$objid);
        } else {
            $errors++;
            # if status is not IN_PROCESS or CONVERTED, set an error 
            update_queue($namespace,$objid,'punted');
            get_logger()->error('GRINError',namespace=>$namespace,objid=>$objid,field=>'conversion',expected=>'Success',
                actual=>$result,detail => 'GRIN refused conversion',stage=>__PACKAGE__);
        }

    }
    get_logger()->info("$namespace GRIN conversion requests: $converted success; $errors failures");
}

# Request conversion for the given barcodes with the given GRIN instance.
# Returns array with hash with barcode & result 
sub grin_convert {
    my $grin_id = shift;
    my $barcodes = shift;
    my $ua = LWP::UserAgent->new();
    my $toreturn = [];

    for(my $i = 0; $i < @$barcodes; $i += $grin_convert_limit) {
        my $last = ($i + ($grin_convert_limit - 1) < $#$barcodes
                        ? $i + ($grin_convert_limit - 1)
                        : $#$barcodes);
        # generate data to post to google: list of uppercased barcodes
        # joined by cgi '&'
        my $post_data = join('&', map { uc($_) } ( @$barcodes[$i..$last] ));
        my $req = HTTP::Request->new(POST => "https://books.google.com/libraries/$grin_id/_process");
        $req->content($post_data);
        my $res = $ua->request($req);
        if($res->is_success) {
        
            foreach my $line (split(/\n/m,$res->content)) {
                chomp $line;
                next if $line =~ /^Barcode/;
                my ($id, $status) = split(/\t/, $line);
                push(@$toreturn, { objid => $id, result => $status });
            }
        } else {
            die($res->status_line);
        }
    }

    return $toreturn;
}


# Query live GRIN status for the given barcodes
# Returns array with hash for each barcode
sub grin_query {
    my $grin_id = shift;
    my $barcodes = shift;
    my $ua = LWP::UserAgent->new();
    my $toreturn = [];
    for(my $i = 0; $i < @$barcodes; $i += $grin_query_limit) {
        my $last = ($i + ($grin_query_limit - 1) < $#$barcodes
                        ? $i + ($grin_query_limit  -1)
                        : $#$barcodes);
        # generate data to post to google: list of uppercased barcodes
        # joined by cgi '&'
        my $barcode_param = join('+', map { uc($_) } ( @$barcodes[$i..$last] ));

        my $req = HTTP::Request->new(GET => 
            "https://books.google.com/libraries/$grin_id/_barcode_search?execute_query=true&barcodes=$barcode_param&format=text&mode=full" );

        my $res = $ua->request($req);
        if($res->is_success) {
        
            foreach my $line (split(/\n/m,$res->content)) {
                chomp $line;
                next if $line =~ /^Barcode/;
                my @fields = split(/\t/, $line);
               # Fields: 
                #    Barcode
                #    Check-In Date
                #    State
                #    Viewability
                #    Conditions
                #    Scannable
                #    Opted-Out (post-scan)
                #    Tagging
                #    Audit
                #    Material Error%
                #    Overall Error%
                #    Scanned Date
                #    Processed Date
                #    Analyzed Date
                #    Converted Date
                #    Allow Download Updated Date
                #    Viewability Updated Date
                #    Source Library Bibkey

                push(@$toreturn, { objid => lc($fields[0]), state => $fields[2] });
            }
        } else {
            die($res->status_line);
        }
    }

    return $toreturn;
}

1;

__END__
