#!/usr/bin/perl

package HTFeed::PackageType::Google::METS;
use strict;
use warnings;
use Log::Log4perl qw(get_logger);
use Date::Manip;
use base qw(HTFeed::METS);

sub _add_header {
    my $self = shift;
    $self->SUPER::_add_header();
    my $volume = $self->{volume};
    $volume->record_premis_event('premis_migration');
    my $xpc = $volume->get_source_mets_xpc();

    # Google: altRecordID handling - reject if there is an altRecordID in the
    # source METS. This should only happen if the volume is a duplicate, which
    # should be detected by looking for condition 31 set and source library
    # bibkey not null, but it doesn't hurt to check.

    foreach my $alt_record_id ($xpc->findnodes('//mets:altRecordID')) {
        $self->set_error("BadFile",file => $volume->get_source_mets_file(),
            field => 'altRecordID',
            value=>$alt_record_id->toString(),
            detail=>'altRecordID found in Google METS');
    }
}

sub _add_schemas {
    my $self = shift;
    $self->SUPER::_add_schemas(@_);

    my $mets = $self->{mets};
    $mets->add_schema( "gbs", "http://books.google.com/gbs");

}

sub _add_source_mets_events {
    my $self = shift;
    my $volume = $self->{volume};
    my $premis = $self->{premis};
    my $xpc = $volume->get_source_mets_xpc();

    my %google_event_map = (
        'Scan' => 'capture',
#        'Analyze' => 'image analysis',
#        'Process' => 'image processing', 
#        'Audit' => 'manual quality review',
#        'Rubbish' => 'partial manual quality review',
    );

    foreach my $event ($xpc->findnodes('//premis:event')) {
        my $google_type = $xpc->findvalue('./premis:eventType',$event);
        if(my $type = $google_event_map{$google_type}) {
            my $eventconfig = {
                type => $type,
                date => $xpc->findvalue('./premis:eventDateTime',$event),
                detail =>  $xpc->findvalue('./premis:eventDetail',$event),
            };
            # Strip off the seconds so that we can reliably match events with ones
            # previously listed via GRIN
            $eventconfig->{date} =~ s/:\d{2}$/:00/;

            my @agents = $xpc->findnodes('./premis:linkingAgentIdentifier',$event);
            if(@agents != 1) {
                $self->set_error("BadField",field => "linkingAgentIdentifier",
                    detail => "Expected 1 linking agent, found " . scalar(@agents));
            }
            my $agent = $agents[0];
            my $agentid = $xpc->findvalue('./premis:linkingAgentIdentifierValue',$agent);
            if($agentid eq 'Google (Firm)') {
                $eventconfig->{'executor_type'} = 'MARC21 Code';
                $eventconfig->{'executor'} = 'Ca-MvGOO';
                # stick a time zone on the date
                if($eventconfig->{date} =~ /^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}$/) {
                    $eventconfig->{date} = $self->convert_tz($eventconfig->{date},'America/Los_Angeles');
                }
            } else {
                $self->set_error("BadField",field=>"linkingAgentIdentifierValue",
                    expected => "Google, Inc. (Firm)", actual => $agentid, 
                    detail => "Unknown agent ID");
            }
            $eventconfig->{'eventid'} =  $volume->make_premis_uuid($type,$eventconfig->{'date'});

            $self->add_premis_event($eventconfig);
        } elsif($google_type eq 'Convert') {
            foreach my $eventcode (('source_mets_creation','page_md5_create','image_compression')) {
                my $eventconfig = $volume->get_nspkg()->get_event_configuration($eventcode);
                $eventconfig->{'date'} = $xpc->findvalue('./premis:eventDateTime',$event);
                $eventconfig->{'date'} =~ s/:\d{2}$/:00/;
                if($eventconfig->{date} =~ /^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}$/) {
                    $eventconfig->{date} = $self->convert_tz($eventconfig->{date},'America/Los_Angeles');
                }
                $eventconfig->{'executor_type'} = 'MARC21 Code';
                $eventconfig->{'executor'} = 'Ca-MvGOO';
                # override tools from config - Google doesn't list any, and the HT tools
                # are not applicable here, just the description
                delete $eventconfig->{'tools'};
                $eventconfig->{'eventid'} = $volume->make_premis_uuid($eventconfig->{'type'},$eventconfig->{'date'});
                $self->add_premis_event($eventconfig);
            }

        }

    }
}

sub _add_techmds {
    my $self = shift;
    my $volume = $self->{volume};
    my $xc = $volume->get_source_mets_xpc();
    $self->SUPER::_add_techmds();

    my $reading_order = new METS::MetadataSection( 'techMD',
        id => $self->_get_subsec_id('TMD'));

    # map from old-style info to new-style info!

    my %page_order_map = (
        # old-style
#        join(';',"left-to-right","non-reading_order","non-reading_order") => ["unknown","unknown","follows-reading-order"], # nonsense; seems that most are ltr
        join(';',"left-to-right","reading-order","reading-order") => ["left-to-right","left-to-right","follows-reading-order"],
#        join(';',"right-to-left","non-reading_order","non-reading_order") => ["unknown","unknown","follows-reading-order"], # nonsense; seems that most are rtl
        join(';',"right-to-left","reading-order","reading-order") => ["left-to-right","right-to-left","follows-reading-order"],
        join(';',"unknown","unknown","unknown") => ["unknown","unknown","follows-reading-order"],

        # new-style
        join(';',"left-to-right","left-to-right","follows-reading-order") => ["left-to-right","left-to-right","follows-reading-order"], # ok as is
#        join(';',"right-to-left","left-to-right","follows-reading-order") => ["unknown","unknown","follows-reading-order"], # nonsense; seems that most are ltr
#        join(';',"left-to-right","right-to-left","follows-reading-order") => ["left-to-right","right-to-left","follows-reading-order"], # ok as is, but not getting this at present
        join(';',"right-to-left","right-to-left","follows-reading-order") => ["left-to-right","right-to-left","follows-reading-order"], # scanning order should be ltr
        join(';',"unknown","unknown","follows-reading-order") => ["unknown","unknown","follows-reading-order"], # ok as is
    );

    

    my @mdwraps = $xc->findnodes('//mets:mdWrap[@LABEL="reading order"]');
    if(@mdwraps == 1) {
        my $mdwrap = $mdwraps[0];
        my $key = join(';',map {$_->textContent()} $xc->findnodes('.//gbs:*',$mdwrap));
        if(defined $page_order_map{$key}) {
            my $values = $page_order_map{$key};

            my $pageOrder= ($xc->findnodes('.//gbs:pageOrder|.//gbs:scanningOrder',$mdwrap))[0];
            $pageOrder->setNodeName('scanningOrder');
            $pageOrder->removeChildNodes();
            $pageOrder->appendTextNode($values->[0]);

            my $pageSequence = ($xc->findnodes('.//gbs:pageSequence|.//gbs:readingOrder',$mdwrap))[0];
            $pageSequence->setNodeName('readingOrder');
            $pageSequence->removeChildNodes();
            $pageSequence->appendTextNode($values->[1]);

            my $coverTag = ($xc->findnodes('.//gbs:coverTag',$mdwrap))[0];
            $coverTag->removeChildNodes();
            $coverTag->appendTextNode($values->[2]);


        } else {
            $self->set_error("BadValue",field=>"gbs:reading order",actual=>$key,detail=>"Don't know how to map reading order");
        }

        $reading_order->set_mdwrap($mdwrap);
        push(@{ $self->{amd_mdsecs} },$reading_order);
    } else {
        my $count = scalar(@mdwraps);
        get_logger->warn("BadField",field=>"reading order",decription=>"Found $count reading order techMDs, expected 1");
    }
}


sub _extract_old_premis {
    my $self = shift;
    my $volume = $self->{volume};
    my $nspkg = $volume->get_nspkg();
    my $xpc = $volume->get_repository_mets_xpc();
    return unless defined $xpc;

    my %premis1_event_map = (
        capture => [qw(capture)],
        # we know the Google source METS & image compression was done at the same time as page md5 creation
        'message digest calculation' => [qw(page_md5_create image_compression)],
        decryption => [qw(decryption)],
        'fixity check' => [qw(page_md5_fixity)],
        'ingestion' => [qw(ingestion zip_compression zip_md5_create)],
        'validation' => [qw(package_validation)]
    );

    # FIXME: $volume->record_premis_event('mets_update');
    my $had_premis1 = 0;
    foreach my $event ($xpc->findnodes('//premis1:event')) {
        $had_premis1 = 1;
        my $premis1_type = $xpc->findvalue('./premis1:eventType',$event);
        if(my $eventcodes = $premis1_event_map{$premis1_type}) {
            foreach my $eventcode (@$eventcodes) {
                my $eventconfig = $nspkg->get_event_configuration($eventcode);
                my $from_tz = undef;
                $eventconfig->{date} = $xpc->findvalue('./premis1:eventDateTime',$event);
                # tool is not represented in PREMIS1; don't make up one
                delete $eventconfig->{'tools'};

                my @agents = $xpc->findnodes('./premis1:linkingAgentIdentifier',$event);
                if(@agents != 1) {
                    $self->set_error("BadField",field => "linkingAgentIdentifier",
                        detail => "Expected 1 linking agent, found " . scalar(@agents));
                }
                my $agent = $agents[0];
                my $agentid = $xpc->findvalue('./premis1:linkingAgentIdentifierValue',$agent);
                if($agentid eq 'Google, Inc.') {
                    $eventconfig->{'executor_type'} = 'MARC21 Code';
                    $eventconfig->{'executor'} = 'Ca-MvGOO';
                    $from_tz = 'America/Los_Angeles';
                } elsif($agentid eq 'UM' or 
                    ($volume->get_namespace() eq 'mdp' 
                            and ($agentid =~ /Michigan/ or $agentid =~ /Trigonix/))) {
                    $eventconfig->{'executor_type'} = 'MARC21 Code';
                    $eventconfig->{'executor'} = 'MiU';
                    # XXX if pulling this from Google METS it may be wrong
                    $from_tz = 'America/Detroit';
                } elsif($agentid eq 'UNKNOWN') {
                    $eventconfig->{'executor_type'} = 'HathiTrust AgentID';
                    $eventconfig->{'executor'} = 'UNKNOWN';
                    $from_tz = 'America/Los_Angeles';
                } else {
                    $self->set_error("BadField",field=>"linkingAgentIdentifierValue",
                        actual => $agentid, 
                        detail => "Unknown agent ID");
                }

                my $outcomeDetail = $xpc->findvalue('.//premis1:eventOutcomeDetail',$event);
                if(defined $outcomeDetail and $outcomeDetail ne '') {
                    my $outcome = PREMIS::Outcome->new($outcomeDetail);
                    $eventconfig->{outcomes} = [$outcome];
                }

                # if date doesn't have a time zone and we know what time zone it should be
                if($eventconfig->{date} =~ /^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}$/ and defined $from_tz) {
                    $eventconfig->{date} = $self->convert_tz($eventconfig->{date},$from_tz);
                } 

                $eventconfig->{'eventid'} =  $volume->make_premis_uuid($eventconfig->{'type'},$eventconfig->{'date'});

                $self->{old_event_types}->{$eventconfig->{type}} = $event;
                $self->add_premis_event($eventconfig);
            }
        }
    }
    $volume->record_premis_event('premis_migration') if($had_premis1);

    # get any PREMIS2 events if they are there..
    return $self->SUPER::_extract_old_premis();
}

1;
